{TMessageHookBasic by Meik Weber
 Version 1.0 (16.05.96)

 English:
 This is Freeware. Modify and use it as you need it.
 This component is delivered AS IS and the author cannot be held responsible for
 any damage caused by using this component.

 You can send a picture postcard to the author if you like this component:
   Meik Weber
   Ostwaldstr. 8
   53115 Bonn
   Germany

 Deutsch:
 Diese Komponente ist Freeware. Sie knnen sie ndern und nutzen, wie Sie wollen.
 Der Autor bernimmt keine Verantwortung fr irgendwelche Schden, die durch
 Einsatz dieser Komponente entstehen.

 Wenn Sie diese Komponente mgen, knnen Sie eine Postkarte (mit Bild) an den
 Autor senden:
   Meik Weber
   Ostwaldstr. 8
   53115 Bonn
   Germany

 }

unit UWMMsg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TMessageHookBasic = class(TComponent)
  private
    { Private declarations }
    FNewWndProc        : Pointer;
    FOldWndProc        : Pointer;
    FAssociatedControl : TWinControl;
    FHooked            : Boolean;
    FControl           : TWinControl;
    FActive            : Boolean;
  protected
    { Protected declarations }
    procedure Notification (AComponent : TComponent; Operation : TOperation); override;
    procedure Loaded; override;
    procedure EstablishHook;
    procedure RemoveHook;
    procedure SetActive  (AValue : Boolean); virtual;
    procedure SetControl (AControl : TWinControl); virtual;
    {override OwnWndProc to provide a complete new handling}
    procedure OwnWndProc (var AMessage : TMessage); virtual;
    {override CustomWndProc to provide a handler for some messages}
    procedure CustomWndProc (var HandleOriginal : Boolean; var AMessage : TMessage); virtual;
  public
    { Public declarations }
    constructor Create (AOwner : TComponent); override;
    destructor  Destroy; override;
    property    Hooked : Boolean read FHooked;
    procedure   OldWndProc (var AMessage : TMessage); virtual;
  published
    { Published declarations }
    property Control   : TWinControl read FControl   write SetControl;
    property Active    : Boolean     read FActive    write SetActive default True;
  end;

implementation

constructor TMessageHookBasic.Create (AOwner : TComponent);
begin
  FHooked  := False;
  FControl := nil;
  FActive  := True;
  if AOwner is TWinControl then
    FAssociatedControl := TWinControl (AOwner);
  inherited Create (AOwner);
  {inherited Create is called after my init, because the Notification method
   is triggerd during this inherited Call. They should state this in the
   help files, as this took me a long time to figure out why my properties were
   wrong during notification}
end;

destructor TMessageHookBasic.Destroy;
begin
  if Hooked then
    RemoveHook;
  if assigned (FNewWndProc) then
    FreeObjectInstance (FNewWndProc);
  inherited Destroy;
end;

procedure TMessageHookBasic.SetActive (AValue : Boolean);
begin
  if csDesigning in ComponentState then
    FActive := AValue
  else if (AValue <> FActive) then begin
    FActive := AValue;
    if AValue then
      EstablishHook
    else
      RemoveHook;
  end;
end;

procedure TMessageHookBasic.SetControl (AControl : TWinControl);
var
  OldActive : Boolean;
begin
  if AControl <> Control then begin
    OldActive := Active;
    Active := False;
    FControl := AControl;
    if not assigned (FControl) then begin
      if Owner is TWinControl then
        FAssociatedControl := TWinControl (Owner)
      else
        FAssociatedControl := nil;
    end else begin
      FAssociatedControl := FControl;
    end;
    Active := OldActive;
  end;
end;

procedure TMessageHookBasic.Notification (AComponent : TComponent; Operation : TOperation);
begin
  inherited Notification (AComponent, Operation);
  if (Operation = opRemove) then
    if AComponent = FAssociatedControl then
      Control := nil
    else if AComponent = Self then
      RemoveHook;
end;

{I hope the following function works and is the correct place to establish
 the hook. I am not sure if the FControl value is always valid at this moment.
 Maybe this is depending on the creation order. For Delphi 2.0 this works.}
procedure TMessageHookBasic.Loaded;
begin
  inherited Loaded;
  EstablishHook;
end;

procedure TMessageHookBasic.EstablishHook;
begin
  if Active and assigned (FAssociatedControl) and not (csDesigning in ComponentState) then begin
    if not assigned (FNewWndProc) then
      FNewWndProc := MakeObjectInstance (OwnWndProc);
    if assigned (FNewWndProc) then begin
      if not Hooked then begin
        FOldWndProc := Pointer (GetWindowLong (FAssociatedControl.Handle, GWL_WNDPROC));
        SetWindowLong (FAssociatedControl.Handle, GWL_WNDPROC, LongInt (FNewWndProc));
      end;  
      FHooked := True;
    end;
  end;
end;  

procedure TMessageHookBasic.RemoveHook;
begin
  if assigned (FNewWndProc) and assigned (FAssociatedControl) and
     FAssociatedControl.HandleAllocated and IsWindow (FAssociatedControl.Handle) then begin
    SetWindowLong (FAssociatedControl.Handle, GWL_WNDPROC, LongInt (FOldWndProc));
    FActive  := False;
    FreeObjectInstance (FNewWndProc);
    FNewWndProc := nil;
    FHooked := False;
  end;
end;

procedure TMessageHookBasic.OwnWndProc (var AMessage : TMessage);
var
  HandleOriginal : Boolean;
begin
  HandleOriginal := True;
  CustomWndProc (HandleOriginal, AMessage);
  if HandleOriginal then
    OldWndProc (AMessage);
end;

procedure TMessageHookBasic.OldWndProc (var AMessage : TMessage);
begin
  if Hooked then
    AMessage.Result :=  CallWindowProc (TFarProc (FOldWndProc), FAssociatedControl.Handle,
                                       AMessage.Msg, AMessage.wParam, AMessage.lParam);
end;

procedure TMessageHookBasic.CustomWndProc (var HandleOriginal : Boolean; var AMessage : TMEssage);
begin
end;

end.
